## FILE GENERATED BY mProv.  Modifications will be overwritten
# Please modify the mProv template.

# <IPv4-address>,<name> ...

{% for host in hosts %}
{% if 'ipaddress' in host and host.ipaddress != "" and host.ipaddress != None %}{{host.ipaddress}} {{host.hostname}} {{host.hostname}}.{{host.domain}} {% endif %}
{% if 'ipv6ll' in host and host.ipv6ll != "" and host.ipv6ll != None %}{{host.ipv6ll}} {{host.hostname}} {{host.hostname}}.{{host.domain}} {% endif %}
{% endfor %}
