#!ipxe
echo "mProv iPXE IP Version Router.
isset ${ip6} && goto ipv6 || goto ipv4 

:ipv4
echo "IPv4 detected."
chain tftp://{{ bootserver }}/menu.ipxe

:ipv6
echo "IPv6 detected."
chain tftp://{{ bootserver6 }}/menu.ipxe